﻿using UnityEngine;
using System.Collections;

public class Turret : MonoBehaviour {
    public float turnSpeed;
    public float viewDistance;
    public LayerMask lookMask;
    public LayerMask shootMask;
    public GameObject bulletPrefab;
    public float shootCooldown;
    public AudioClip shootSound;

    private GameObject player;
    private LineRenderer line;
    private Vector3 lastSeen;
    private float lastDistance;
    private float cooldownTimer = 0;
    private AudioSource aSource;

	// Use this for initialization
	void Start () {
        player = GameObject.FindGameObjectWithTag("Player");
        line = GetComponentInChildren<LineRenderer>();
        aSource = GetComponent<AudioSource>();

        lastSeen = Vector3.Normalize(player.transform.position - transform.position);
        lastDistance = Vector3.Distance(player.transform.position, transform.position);
    }
	
	// Update is called once per frame
	void Update () {
        if (cooldownTimer > 0)
        {
            cooldownTimer -= Time.deltaTime;
            line.enabled = false;
        }
        else
        {
            line.enabled = true;
        }
	}

    void FixedUpdate()
    {
        Vector3 direction = lastSeen;
        float distance = lastDistance;
        if (player != null)
        {
            direction = Vector3.Normalize(player.transform.position - transform.position);
            distance = Vector3.Distance(player.transform.position, transform.position);
        }
        if (Physics.Raycast(transform.position, direction, distance, lookMask) == false & distance <= viewDistance)
        {
            lastSeen = direction;
            lastDistance = distance;
            rotateTowards(direction);

            if (Physics.Raycast(transform.position, transform.forward, viewDistance, shootMask) == true)
            {
                if (cooldownTimer <= 0)
                {
                    Vector3 bSpawnPos = transform.position;
                    bSpawnPos += transform.forward * 1;
                    GameObject bullet = Object.Instantiate(bulletPrefab);
                    bullet.transform.position = bSpawnPos;
                    bullet.transform.eulerAngles = transform.eulerAngles;

                    aSource.PlayOneShot(shootSound);

                    cooldownTimer = shootCooldown;
                }
            }
        }
        else
        {
            rotateTowards(lastSeen);
        }

        RaycastHit hit;
        if (Physics.Raycast(transform.position, transform.forward, out hit, Mathf.Infinity) == true)
        {
            line.SetPosition(0, transform.position);
            line.SetPosition(1, hit.point);
        }
    }

    private void rotateTowards(Vector3 direction)
    {
        if (cooldownTimer <= 0)
        {
            Vector3 rotate = Vector3.RotateTowards(transform.forward, direction, turnSpeed * Time.deltaTime, 0.0f);
            transform.rotation = Quaternion.LookRotation(rotate);
        }
    }
}
